/***************************************************************************************
 * @file     va108xx_hal_i2c.h
 * @version  V1.0
 * @date     31 October 2018
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2018 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
#ifndef __HAL_I2C_H
#define __HAL_I2C_H

/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include "va108xx_hal.h"
#include "board.h"
#include "va108xx_debug.h"

/*****************************************************************************/ 
/* Global pre-processor symbols/macros ('#define')                           */ 
/*****************************************************************************/

/** Version */
#define HAL_I2C_VERSION (0x00000300) // 0.3.0 (05 June 2019)

/*****************************************************************************/
/* Global type definitions ('typedef')                                       */ 
/*****************************************************************************/

//"shifted" means bit[0] is already the placeholder for the Read/Write_N bit
//do not put a '1' in Read/Write_N bit
//Write does not strip it, but Read ORs it in
typedef uint32_t i2caddr7_shifted_t;
typedef uint32_t i2caddr_shifted_t;

enum I2C_ERR_ENUM {
	I2C_ERR_NULL      =0,
	I2C_ERR_BUSNOTIDLE=1,
	I2C_ERR_ADDRSTUCK =2,
	I2C_ERR_NACKADDR  =3,
	I2C_ERR_TXDSTUCK  =4,
	I2C_ERR_CMDSTUCK  =5,
	I2C_ERR_RXOVERRUN =6,
	I2C_ERR_RXSTALL   =7,
	I2C_ERR_RXCOUNT   =8
};

/*****************************************************************************/
/* Global variable declarations ('extern', definition in C source)           */
/*****************************************************************************/

/*****************************************************************************/ 
/* Global function prototypes ('extern', definition in C source)             */ 
/*****************************************************************************/

// I2CA Slave
extern hal_status_t I2C_I2CA_SlaveInit(i2caddr_shifted_t i2cAddr);

// I2CB Slave
extern hal_status_t I2C_I2CB_SlaveInit(i2caddr_shifted_t i2cAddr);

// I2CA Master
extern hal_status_t I2C_I2CA_MasterInit(void);
extern hal_status_t I2C_I2CA_MasterRead(i2caddr_shifted_t i2cAddr_preshift, char *dataPtr, uint32_t len);
extern hal_status_t I2C_I2CA_MasterWrite(i2caddr_shifted_t i2cAddr_preshift, char *dataPtr, uint32_t len);

// I2CB Master
extern hal_status_t I2C_I2CB_MasterInit(void);
extern hal_status_t I2C_I2CB_MasterRead(i2caddr_shifted_t i2cAddr_preshift, char *dataPtr, uint32_t len);
extern hal_status_t I2C_I2CB_MasterWrite(i2caddr_shifted_t i2cAddr_preshift, char *dataPtr, uint32_t len);

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
#endif /* __HAL_I2C_H */
