/***************************************************************************************
 * @file     va108xx_hal_i2c.c
 * @version  V0.3
 * @date     31 October 2018
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2018 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include "va108xx_hal_i2c.h"
#include "va108xx_hal_timer.h"

/*****************************************************************************/ 
/* Local pre-processor symbols/macros ('#define')                            */ 
/*****************************************************************************/

#define I2C_TIMEOUT 0x10000000
#define I2C_FIFO_DEPTH (16)

#ifndef I2C_TIMEOUT_TIM
#define I2C_TIMEOUT_TIM (0)
#endif

#define TIMEOUT_EXPIRED (0 == (VOR_TIM->BANK[I2C_TIMEOUT_TIM].CTRL & TIM_PERIPHERAL_CTRL_ACTIVE_Msk))

/*****************************************************************************/ 
/* Global variable definitions (declared in header file with 'extern')       */ 
/*****************************************************************************/

/*****************************************************************************/ 
/* Local type definitions ('typedef')                                        */ 
/*****************************************************************************/

/*****************************************************************************/ 
/* Local variable definitions ('static')                                     */ 
/*****************************************************************************/

/*****************************************************************************/ 
/* Local function prototypes ('static')                                      */ 
/*****************************************************************************/

/*****************************************************************************/ 
/* Function implementation - global ('extern') and local ('static')          */ 
/*****************************************************************************/

hal_status_t I2C_I2CA_SlaveInit(i2caddr_shifted_t i2cAddr)
{
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_I2CA;
  VOR_SYSCONFIG->PERIPHERAL_RESET &= ~(CLK_ENABLE_I2CA);
  __NOP();
  __NOP();
  VOR_SYSCONFIG->PERIPHERAL_RESET |= CLK_ENABLE_I2CA;
	VOR_I2CA->S0_CTRL=(I2CB_S0_CTRL_ENABLE_Msk);
	VOR_I2CA->CLKSCALE=0x0018; // =/25 for 50MHz
	VOR_I2CA->S0_ADDRESS=i2cAddr | 0x1; //preshifted
  	
	VOR_I2CA->FIFO_CLR=I2C_PERIPHERAL_S0_FIFO_CLR_RXFIFO_Msk | I2C_PERIPHERAL_S0_FIFO_CLR_TXFIFO_Msk;
  return hal_status_ok;
}

hal_status_t I2C_I2CB_SlaveInit(i2caddr_shifted_t i2cAddr)
{
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_I2CB;
  VOR_SYSCONFIG->PERIPHERAL_RESET &= ~(CLK_ENABLE_I2CB);
  __NOP();
  __NOP();
  VOR_SYSCONFIG->PERIPHERAL_RESET |= CLK_ENABLE_I2CB;
	VOR_I2CB->S0_CTRL=(I2CB_S0_CTRL_ENABLE_Msk);
	VOR_I2CB->CLKSCALE=0x0018; // =/25 for 50MHz
	VOR_I2CB->S0_ADDRESS=i2cAddr | 0x1; //preshifted
  	
	VOR_I2CB->FIFO_CLR=I2C_PERIPHERAL_S0_FIFO_CLR_RXFIFO_Msk | I2C_PERIPHERAL_S0_FIFO_CLR_TXFIFO_Msk;
  return hal_status_ok;
}

hal_status_t I2C_I2CA_MasterInit(void)
{
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_I2CA;
  VOR_SYSCONFIG->PERIPHERAL_RESET &= ~(CLK_ENABLE_I2CA);
  __NOP();
  __NOP();
  VOR_SYSCONFIG->PERIPHERAL_RESET |= CLK_ENABLE_I2CA;
	VOR_I2CA->CLKSCALE=0x0018; // =/25 for 50MHz
	VOR_I2CA->FIFO_CLR=I2C_PERIPHERAL_FIFO_CLR_RXFIFO_Msk | I2C_PERIPHERAL_FIFO_CLR_TXFIFO_Msk;
	VOR_I2CA->CTRL=(I2CA_CTRL_DLGFILTER_Msk | I2CA_CTRL_ENABLE_Msk);
  return hal_status_ok;
}

hal_status_t I2C_I2CB_MasterInit(void)
{
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_I2CB;
  VOR_SYSCONFIG->PERIPHERAL_RESET &= ~(CLK_ENABLE_I2CB);
  __NOP();
  __NOP();
  VOR_SYSCONFIG->PERIPHERAL_RESET |= CLK_ENABLE_I2CB;
	VOR_I2CB->CLKSCALE=0x0018; // =/25 for 50MHz
	VOR_I2CB->FIFO_CLR=I2C_PERIPHERAL_FIFO_CLR_RXFIFO_Msk | I2C_PERIPHERAL_FIFO_CLR_TXFIFO_Msk;
	VOR_I2CB->CTRL=(I2CB_CTRL_DLGFILTER_Msk | I2CB_CTRL_ENABLE_Msk);
  return hal_status_ok;
}

hal_status_t I2C_I2CA_MasterWrite(i2caddr_shifted_t i2cAddr_preshift,char *dataPtr, uint32_t len)
{
	if(!len){
		return hal_status_badParam;
  }
  VOR_I2CA->WORDS=len;
	VOR_I2CA->ADDRESS=i2cAddr_preshift;
	while(len--){
     VOR_I2CA->DATA=*dataPtr++;
	}
	VOR_I2CA->CMD=0x03;//start with stop
  HAL_Timer_SetupTimeoutMs(I2C_TIMEOUT_TIM, 100);
	while((VOR_I2CA->STATUS & I2CA_STATUS_IDLE_Msk) == 0)
	{
    if(TIMEOUT_EXPIRED){
      return hal_status_timeout;
    }
	}
  if((VOR_I2CA->STATUS & (I2CA_STATUS_NACKADDR_Msk | I2CA_STATUS_NACKDATA_Msk)) != 0){
    return hal_status_nak;
  }
  return hal_status_ok;
}

hal_status_t I2C_I2CB_MasterWrite(i2caddr_shifted_t i2cAddr_preshift,char *dataPtr, uint32_t len)
{
	if(!len){
		return hal_status_badParam;
  }
  VOR_I2CB->WORDS=len;
	VOR_I2CB->ADDRESS=i2cAddr_preshift;
	while(len--){
     VOR_I2CB->DATA=*dataPtr++;
	}
	VOR_I2CB->CMD=0x03;//start with stop
  HAL_Timer_SetupTimeoutMs(I2C_TIMEOUT_TIM, 100);
	while((VOR_I2CB->STATUS & I2CB_STATUS_IDLE_Msk) == 0)
	{
    if(TIMEOUT_EXPIRED){
      return hal_status_timeout;
    }
	}
	if((VOR_I2CB->STATUS & (I2CB_STATUS_NACKADDR_Msk | I2CB_STATUS_NACKDATA_Msk)) != 0){
    return hal_status_nak;
  }
  return hal_status_ok;
}

hal_status_t I2C_I2CA_MasterRead(i2caddr_shifted_t i2cAddr_preshift, char *dataPtr, uint32_t len)
{
	volatile uint32_t result;
  
  if(!len){
		return hal_status_badParam;
  }
  
	VOR_I2CA->WORDS=len;
	VOR_I2CA->ADDRESS=i2cAddr_preshift | 0x01;
	VOR_I2CA->CMD=0x03; //start with stop
	
  HAL_Timer_SetupTimeoutMs(I2C_TIMEOUT_TIM, 100);
	while((VOR_I2CA->STATUS & I2CA_STATUS_IDLE_Msk)==0)// wait for IDLE condition
  {
		if(TIMEOUT_EXPIRED){
      return hal_status_timeout;
    }
	}
	
	while(len--){
    HAL_Timer_SetupTimeoutMs(I2C_TIMEOUT_TIM, 100);
		while(!(VOR_I2CA->STATUS & I2CA_STATUS_RXNEMPTY_Msk)){
			if(TIMEOUT_EXPIRED){
        return hal_status_timeout;
      }
		}
	  *dataPtr++ = VOR_I2CA->DATA;
	}
  return hal_status_ok;
}

hal_status_t I2C_I2CB_MasterRead(i2caddr_shifted_t i2cAddr_preshift, char *dataPtr, uint32_t len)
{
	volatile uint32_t result;
  
  if(!len){
		return hal_status_badParam;
  }
  
	VOR_I2CB->WORDS=2;//read 2 bytes
	VOR_I2CB->ADDRESS=i2cAddr_preshift | 0x01;
	VOR_I2CB->CMD=0x03; //start with stop
  
  HAL_Timer_SetupTimeoutMs(I2C_TIMEOUT_TIM, 100);
	while((VOR_I2CB->STATUS & I2CB_STATUS_IDLE_Msk)==0)// wait for IDLE condition
  {
		if(TIMEOUT_EXPIRED){
      return hal_status_timeout;
    }
	}
  
  while(len--){
    HAL_Timer_SetupTimeoutMs(I2C_TIMEOUT_TIM, 100);
		while(!(VOR_I2CB->STATUS & I2CB_STATUS_RXNEMPTY_Msk)){
			if(TIMEOUT_EXPIRED){
        return hal_status_timeout;
      }
		}
	  *dataPtr++ = VOR_I2CB->DATA;
	}
  return hal_status_ok;
}

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
